/**
 * Copyright 2010 The Regents of the University of California
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on
 * an "AS IS"; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under
 * the License.
 * 
 * Author: Rares Vernica <rares (at) ics.uci.edu>
 */

package edu.uci.ics.fuzzyjoin.similarity;

import edu.uci.ics.fuzzyjoin.tokenizer.Tokenizer;

public abstract class SimilarityMetric {

    public static int getIntersectSize(int[] tokensX, int startX,
            int[] tokensY, int startY) {
        int intersectSize = 0;

        while (startX < tokensX.length && startY < tokensY.length) {
            int tokenX = tokensX[startX];
            int tokenY = tokensY[startY];
            if (tokenX > tokenY) {
                startY++;
            } else if (tokenX < tokenY) {
                startX++;
            } else {
                intersectSize++;
                startX++;
                startY++;
            }
        }

        return intersectSize;
    }

    public static int getIntersectSize(int[] tokensX, int[] tokensY) {
        return getIntersectSize(tokensX, 0, tokensY, 0);
    }

    public static PartialIntersect getPartialIntersectSize(int[] tokensX,
            int[] tokensY, int tokenStop) {
        int startX = 0;
        int startY = 0;
        int intersectSize = 0;
        PartialIntersect p = null;

        while (startX < tokensX.length && startY < tokensY.length) {
            int tokenX = tokensX[startX];
            int tokenY = tokensY[startY];
            if (tokenX > tokenY) {
                startY++;
            } else if (tokenX < tokenY) {
                startX++;
            } else {
                intersectSize++;
                if (p == null) {
                    p = new PartialIntersect();
                    p.posXStart = startX;
                    p.posYStart = startY;
                }
                if (tokenX == tokenStop) {
                    p.posXStop = startX;
                    p.posYStop = startY;
                    p.intersectSize = intersectSize;
                    return p;
                }
                startX++;
                startY++;
            }
        }

        return null;
    }

    // @SuppressWarnings("unchecked")
    // public static int getIntersectSize(DataBag tokensX, DataBag tokensY) {
    // int intersectSize = 0;
    //
    // Iterator<Tuple> iteratorX = tokensX.iterator();
    // Iterator<Tuple> iteratorY = tokensY.iterator();
    //
    // Tuple nextX = null;
    // Tuple nextY = null;
    //
    // while ((nextX != null || iteratorX.hasNext())
    // && (nextY != null || iteratorY.hasNext())) {
    // if (nextX == null) {
    // nextX = iteratorX.next();
    // }
    // if (nextY == null) {
    // nextY = iteratorY.next();
    // }
    //
    // int cmp = nextX.compareTo(nextY);
    // if (cmp > 0) {
    // nextY = null;
    // } else if (cmp < 0) {
    // nextX = null;
    // } else {
    // intersectSize++;
    // nextX = null;
    // nextY = null;
    // }
    // }
    //
    // return intersectSize;
    // }

    // public abstract float getSimilarity(DataBag tokensX, DataBag tokensY);

    // public abstract float getSimilarity(DataBag tokensX, int lengthX,
    // DataBag tokensY, int lengthY);

    public abstract float getSimilarity(int[] tokensX, int lengthX,
            int[] tokensY, int lengthY);

    public abstract float getSimilarity(int[] tokensX, int[] tokensY);

    public abstract float getSimilarity(String stringX, String stringY,
            Tokenizer tokenizer);
}
